<?php

// $Horde: gollem/lib/Driver/ftp.php 06/24/2002 1:41:01 mike $

require_once GOLLEM_BASE . '/lib/constants.php';
require_once HORDE_BASE . '/lib/Prefs.php';

/*
 * FTP Driver implementation for Gollem's Driver abstraction.
 * 
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Michael Varghese <mike.varghese@ascellatech.com>
 * @version <INSERT REVISION>
 * @since   Gollem
 * @package gollem
 */
class Gollem_Driver_ftp extends Gollem_Driver
{	
	function Gollem_Driver_ftp($params)
	{
		$this->type = 'ftp';
        $this->params = $params;
	}
	
	/**
     * Take posted from the login screen and attempt to set up an
     * initial Gollem session. This function should only be called
     * once, when the user first logs in.
     *
     * @access public
     */
	function createSession()
	{
        global $gollem, $conf, $prefs, $registry;
		
        if (!isset($_POST['fileuser'])
            || !isset($_POST['pass'])
            || !isset($_POST['server'])) {
            return false;
        }
        
        $gollem = array();
        
        $gollem['user'] = trim($_POST['fileuser']);
        $gollem['pass'] = Secret::write(Secret::getKey('gollem'), trim($_POST['pass']));
        
        /* We might need to override some of the defaults with
           environmental settings. */
        if ($conf['server']['server_list']) {
            include_once GOLLEM_BASE . '/config/servers.php';
        }
        
        include_once HORDE_BASE . '/lib/Auth.php';
        if (Auth::getAuth()) {
            $gollem['uniquser'] = Auth::getAuth();
        } elseif ($conf['server']['server_list'] && !empty($servers[$_POST['server']]['realm'])) {
            $gollem['uniquser'] = $gollem['user'] . '@' . $servers[$_POST['server']]['realm'];
        } else {
            $gollem['uniquser'] = $gollem['user'];
        }
        
        if ($conf['server']['server_list']
            && isset($servers[$_POST['server']])
            && is_array($servers[$_POST['server']])) {
            $svr = &$_POST['server'];
            $gollem['server'] = $servers[$svr]['server'];
            $gollem['port'] = $servers[$svr]['port'];
            if (isset($conf['server']['show_dotfiles'])) {
                $gollem['show_dotfiles'] = $conf['server']['show_dotfiles'];
            }
            if (isset($servers[$svr]['show_dotfiles'])) {
                $gollem['show_dotfiles'] = $servers[$svr]['show_dotfiles'];
            }
        } else {
            $gollem['server'] = Horde::getFormData('server');
            $gollem['port'] = Horde::getFormData('port');
            $gollem['show_dotfiles'] = $conf['server']['show_dotfiles'];
        }

        $GLOBALS['_SESSION']['gollem'] = &$gollem;
        Gollem_Driver_ftp::setupSession();
        session_register('gollem');
        
        return true;
	}
	
	/**
     * Set up the session for use in the page. Retrieve preferences,
     * and make sure to update the sorting information, if it has
     * been changed.
     *
     * @access public
     */
	function setupSession()
	{
		global $conf, $prefs, $registry;
        
        if (!isset($GLOBALS['_SESSION']['gollem'])) {
            return false;
        } elseif (!isset($GLOBALS['gollem'])) {
            $GLOBALS['gollem'] = &$GLOBALS['_SESSION']['gollem'];
        }
        $sortby = Horde::getFormData('sortby', 0);
		$sortdir = Horde::getFormData('sortdir', 0);
		
        if (isset($sortby)) {
            $prefs->setValue('sortby', Horde::getFormData('sortby'));
        }
        if (isset($sortdir)) {
           $prefs->setValue('sortdir', Horde::getFormData('sortdir'));
        }
        
        return true;
	}
	
	/**
     * Attempt to open a connection to the FTP server using the
     * information in the $gollem session variable.
     *
     * @access public
     *
     * @return bool $ok   Success or failure.
     */
	function authenticate()
	{
	 global $gollem, $conf;
        
        if (!(isset($gollem) && is_array($gollem))) {
            return false;
        }

        $gollem['stream'] = @ftp_connect($gollem['server'], $gollem['port']);

        if (!$gollem['stream'] ||
            !@ftp_login($gollem['stream'], $gollem['user'], Secret::read(Secret::getKey('gollem'), $gollem['pass'])) ||
            !@ftp_pasv($gollem['stream'], true)) {
            if (!empty($gollem['server']) && !empty($gollem['port']) &&
                !empty($gollem['user'])) {
                if (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
                    $entry = sprintf('FAILED LOGIN %s (forwarded for [%s]) to %s:%s as %s',
                                     $_SERVER['REMOTE_ADDR'], $_SERVER['HTTP_X_FORWARDED_FOR'], 
                                     $gollem['server'], $gollem['port'], $gollem['user'],'');
                } else {
                    $entry = sprintf('FAILED LOGIN %s to %s:%s as %s',
                                     $_SERVER['REMOTE_ADDR'], $gollem['server'], $gollem['port'], $gollem['user']);
                }
                Horde::logMessage($entry, __FILE__, __LINE__, LOG_INFO);
            }
            
            $gollem = null;
            session_unregister('gollem');
            return false;
        }
        
        /* Navigate to dir and get the file list from ftp */
        $dir = Horde::getFormData('dir');
        if ($dir !== null) {
            $gollem['dir'] = $dir;
        } else if (isset($GLOBALS['_SESSION']['gollem']['dir'])) {
            $gollem['dir'] = $GLOBALS['_SESSION']['gollem']['dir'];
        }
        
        if (!isset($gollem['dir'])) $gollem['dir'] = '';
        
        if (!empty($gollem['dir'])) {
            if (!@ftp_chdir($gollem['stream'], $gollem['dir'])) {
                $notification->push(_("No such directory: ") . $gollem['dir'], 'horde.warning');
                $gollem['dir'] = '';
            }
        }

        Gollem_Driver_ftp::setLabel();
        
        return true;
	}
	
	function getFileList($dir = '')
	{
	 global $gollem, $prefs;
        
        $files = array();
        $type = @ftp_systype($gollem['stream']);
        $list = @ftp_rawlist($gollem['stream'], $dir);
        
        if (!is_array($list)) {
            return array();
        }
        
        foreach ($list as $line) {
            $file = array();
            $item = preg_split('/\s+/', $line);
            if ($type == 'UNIX' || ($type == 'Windows_NT' && !preg_match('|\d\d-\d\d-\d\d|', $item[0]))) {
                if (count($item) < 7 || substr($line, 0, 5) == 'total') {
                    continue;
                }

                $file['perms'] = $item[0];
                $file['owner'] = $item[2];
                $file['group'] = $item[3];
                $file['name'] = substr($line, strpos($line, sprintf("%s %2s %5s", $item[5], $item[6], $item[7])) + 13);
                if (preg_match('/^\.\.?\/?$/', $file['name'])) {
                    continue;
                }
                if (empty($GLOBALS['gollem']['show_dotfiles']) && (substr($file['name'], 0, 1) === '.')) {
                    continue;
                }
                $p1 = substr($file['perms'], 0, 1);
                if ($p1 === 'l') {
                    $file['link'] = substr($file['name'], strpos($file['name'], '->') + 3);
                    $file['name'] = substr($file['name'], 0, strpos($file['name'], '->') - 1);
                    $file['type'] = '**sym';
                } else if ($p1 === 'd') {
                    $file['type'] = '**dir';
                } else {
                    $name = explode('.', $file['name']);
                    if (count($name) == 1 || (substr($file['name'], 0, 1) === '.' && count($name) == 2)) {
                        $file['type'] = '**none';
                    } else {
                        $file['type'] = strtolower($name[count($name) - 1]);
                    }
                }
                
                if ($file['type'] == '**dir') {
                    $file['size'] = -1;
                } else {
                    $file['size'] = $item[4];
                }
                if (strstr($item[7], ':')) {
                    $file['date'] = strtotime($item[7] . ':00' . $item[5] . ' ' . $item[6] . ' ' . date('Y', time()));
                    if ($file['date'] > time()) {
                        $file['date'] = strtotime($item[7] . ':00' . $item[5] . ' ' . $item[6] . ' ' . (date('Y', time()) - 1));
                    }
                } else {
                    $file['date'] = strtotime('00:00:00' . $item[5] . ' ' . $item[6] . ' ' . $item[7]);
                }
            } else {
                /* Handle Windows FTP servers returning DOS-style file listings. */
                $file['perms'] = '';
                $file['owner'] = '';
                $file['group'] = '';
                $file['name'] = $item[3];
                $index = 4;
                while ($index < count($item)) {
                    $file['name'] .= ' ' . $item[$index];
                    $index++;
                }
                $file['date'] = strtotime($item[0] . ' ' . $item[1]);
                if ($item[2] == '<DIR>') {
                    $file['type'] = '**dir';
                    $file['size'] = -1;
                } else {
                    $file['size'] = $item[2];
                    $name = explode('.', $file['name']);
                    if (count($name) == 1 || (substr($file['name'], 0, 1) === '.' && count($name) == 2)) {
                        $file['type'] = '**none';
                    } else {
                        $file['type'] = strtolower($name[count($name) - 1]);
                    }
                }
            }
            
            $files[$file['name']] = $file;
        }
        
        /* Figure out the real type of symlinks. */
        foreach ($files as $key => $file) {
            if ($file['type'] === '**sym') {
                if (isset($files[$file['link']]) && ($file['name'] !== $file['link'])) {
                    $file['type'] = $files[$file['link']]['type'];
                } else {
                    $file['type'] = '**broken';
                }
                $files[$key] = $file;
            }
        }
        
        switch ($prefs->getValue('sortby')) {
        case SORT_TYPE:
            usort($files, array('Gollem_Driver_ftp', 'sortType'));
            break;
            
        case SORT_NAME:
            usort($files, array('Gollem_Driver_ftp', 'sortName'));
            break;
            
        case SORT_DATE:
            usort($files, array('Gollem_Driver_ftp', 'sortDate'));
            break;
            
        case SORT_SIZE:
            usort($files, array('Gollem_Driver_ftp', 'sortSize'));
            break;
        }
        
        return $files;
	}
	
	function sortDate($a, $b)
    {
       global $prefs;
        
        if ($a['date'] > $b['date']) {
            return $prefs->getValue('sortdir') ? -1 : 1;
        } elseif ($a['date'] === $b['date']) {
            return Gollem_Driver_ftp::sortName($a, $b);
        } else {
            return $prefs->getValue('sortdir') ? 1 : -1;
        }
    }
    
    function sortSize($a, $b)
    {
        global $prefs;
        
        if ($a['size'] > $b['size']) {
            return $prefs->getValue('sortdir') ? -1 : 1;
        } elseif ($a['size'] === $b['size']) {
            return 0;
        } else {
            return $prefs->getValue('sortdir') ? 1 : -1;
        }
    }
    
    function sortType($a, $b)
    {
        global $prefs;
        
        if ($a['type'] === $b['type']) {
            return Gollem_Driver_ftp::sortName($a, $b);
        } elseif ($a['type'] === '**dir') {
            return $prefs->getValue('sortdir') ? 1 : -1;
        } elseif ($b['type'] === '**dir') {
            return $prefs->getValue('sortdir') ? -1 : 1;
        } else {
            $res = strcasecmp($a['type'], $b['type']);
            return $prefs->getValue('sortdir') ? ($res * -1) : $res;
        }
    }
    
    function sortName($a, $b)
    {
        global $prefs;
        
        $res = strcasecmp($a['name'], $b['name']);
        return $prefs->getValue('sortdir') ? ($res * -1) : $res;
    }
	
	function setLabel()
	{
		$GLOBALS['gollem']['label'] = !empty($GLOBALS['gollem']['dir']) ? $GLOBALS['gollem']['dir'] : _("[home]");
	}
	
	function createFolder($name)
	{
		global $gollem;
		return (@ftp_mkdir($gollem['stream'], $name));
	}
	
	function renameItem($old, $new)
	{
		global $gollem;
		return (@ftp_rename($gollem['stream'], $old, $new));
	}
	
	function deleteFolder($name)
	{
		global $gollem;
		return(@ftp_rmdir($gollem['stream'], $name));
	}
	
	function deleteFile($name)
	{
		global $gollem;
		return(@ftp_delete($gollem['stream'], $name));
	}

	function modifyCHMOD($chmod, $name)
	{
		global $gollem;
		return(@ftp_site($gollem['stream'], 'CHMOD' . ' ' . $chmod . ' ' . $name));
	}

	function uploadFile($name, $safe_file, $uploadFormat)
	{
		global $gollem;
		return(@ftp_put($gollem['stream'], $name, $safe_file, $uploadFormat));
	}
	function quit()
	{
		global $gollem;
		@ftp_quit($gollem['stream']);
	}
}

?>